package ifx.smb;

import java.io.*;
import java.net.UnknownHostException;

import jcifs.*;
import jcifs.smb.*;
import jcifs.util.Log;

class SmbTest
{
	private SmbTest()
	{
        String domain = null;
		String user = null;
		String password = null;

        UniAddress dc = null;
        NtlmPasswordAuthentication ntlmP = null;
        NtlmPasswordAuthentication ntlmH = null;
        byte[] challenge = null;
        byte[] ansiHash = null;
        byte[] unicodeHash = null;

		try
		{
			BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
			ConsoleEraser consoleEraser = new ConsoleEraser();

			domain = "EU";
			System.out.println("Domain:   " + domain);

			//System.out.print("Domain:   ");
			//domain = stdin.readLine();

			user = System.getProperty("user.name");
			System.out.println("User:     " + user);

			//System.out.print("User:     ");
			//user = stdin.readLine();

			System.out.print("Password: ");
			consoleEraser.start();
			password = stdin.readLine();
			consoleEraser.halt();
			System.out.print("\b");

			stdin.close();
		}
		catch (IOException ex) {};

        Config.setProperty("jcifs.smb.client.soTimeout", "0");
        Config.setProperty("jcifs.netbios.cachePolicy", "0");

        System.out.println();
        System.out.println("== dc lookup");
        try
        {
            dc = UniAddress.getByName(domain, true);
        }
        catch(UnknownHostException uhe)
        {
            System.out.println("-- unknown host exception [getByName]");
        }
        System.out.println(dc);

		ntlmP = new NtlmPasswordAuthentication(domain, user, password);

		System.out.println();
		System.out.println("== getting challenge from dc");
		try
		{
            challenge = SmbSession.getChallenge(dc);
        }
		catch(SmbException se)
		{
		    System.out.println("-- smb exception [getChallenge]");
		}
        catch(UnknownHostException uhe)
        {
            System.out.println("-- unknown host exception [getChallenge]");
        }
        Log.printHexDump(System.out, challenge, 0, challenge.length);

		System.out.println("== ansi hash (LM hash)");
		ansiHash = ntlmP.getAnsiHash(challenge);
        Log.printHexDump(System.out, ansiHash, 0, ansiHash.length);

		System.out.println("== unicode hash (NT hash)");
		unicodeHash = ntlmP.getUnicodeHash(challenge);
        Log.printHexDump(System.out, unicodeHash, 0, unicodeHash.length);

		ntlmH = new NtlmPasswordAuthentication(domain, user, ansiHash, unicodeHash);

        System.out.println("== trying to login... [user, ansiHash, unicodeHash]");
		try
		{
		    SmbSession.logon(dc, ntlmH);
		    //SmbSession.logonLogoff(dc, ntlmH);
		    System.out.println("login OK");
		}
		catch(SmbAuthException sae)
		{
		    System.out.println("login FAILED");
		}
		catch(SmbException se)
		{
		    System.out.println("-- smb exception [logon]");
		}

		System.out.println();
		System.out.println("== ansi hash (LM hash)");
        for (int i = 0; i < ansiHash.length; i++)
		{
		    ansiHash[i] = 0;
		}
        Log.printHexDump(System.out, ansiHash, 0, ansiHash.length);

		System.out.println("== unicode hash (NT hash)");
		for (int i = 0; i < unicodeHash.length; i++)
		{
		    unicodeHash[i] = 0;
		}
        Log.printHexDump(System.out, unicodeHash, 0, unicodeHash.length);

		ntlmH = new NtlmPasswordAuthentication(domain, user, ansiHash, unicodeHash);

        System.out.println("== trying to login... [user, ansiHash, unicodeHash]");
		try
		{
		    SmbSession.logon(dc, ntlmH);
		    //SmbSession.logonLogoff(dc, ntlmH);
		    System.out.println("login OK");
		}
		catch(SmbAuthException sae)
		{
		    System.out.println("login FAILED");
		}
		catch(SmbException se)
		{
		    System.out.println("-- smb exception [logon]");
		}
	}

	public static void main(String args[])
	{
		SmbTest instance = new SmbTest();
	}
}

class ConsoleEraser extends Thread
{
 	private boolean running = true;

 	public void run()
 	{
  		while (running)
  		{
  			// overwrite console echo with blanks (for password entry)
   			System.out.print("\b ");
   		}
 	}

 	public synchronized void halt()
 	{
  		running = false;
 	}
}